var SequenceStatus;
(function (SequenceStatus) {
    SequenceStatus[SequenceStatus["IDLE"] = 0] = "IDLE";
    SequenceStatus[SequenceStatus["RUNNING"] = 1] = "RUNNING";
    SequenceStatus[SequenceStatus["PAUSED"] = 2] = "PAUSED";
})(SequenceStatus || (SequenceStatus = {}));
class Display {
    constructor() {
        this.sequence = false;
        this.offsetX = 0;
        this.offsetY = 0;
        this.width = 0;
        this.height = 0;
        this.canvasScale = 0;
        this.canvasWidth = 0;
        this.canvasHeight = 0;
        this.canvasOffsetX = 0;
        this.canvasOffsetY = 0;
        this.screenWidth = 0;
        this.screenHeight = 0;
        this.screenOffsetX = 0;
        this.screenOffsetY = 0;
        this.displayWidth = 0;
        this.displayHeight = 0;
        this.displayOffsetX = 0;
        this.displayOffsetY = 0;
        this.parentElement = document.getElementById('display');
        this.create();
        window.onresize = this.onResize.bind(this);
    }
    create() {
        this.canvas = this.parentElement.getElementsByTagName('canvas')[0];
        this.ctx = this.canvas.getContext('2d');
        this.screen = {
            width: parseInt(document.getElementById('width').value),
            height: parseInt(document.getElementById('height').value)
        };
        this.updateSize();
        this.clear();
        this.updateScreen();
        this.updateDisplay();
    }
    updateSize() {
        this.canvasScale = window.devicePixelRatio;
        this.canvasWidth = this.parentElement.clientWidth - 12;
        this.canvasHeight = this.parentElement.clientHeight - 12;
        console.log(`${this.canvasWidth},${this.canvasHeight}`);
        this.canvas.width = this.canvasWidth;
        this.canvas.height = this.canvasHeight;
    }
    clear() {
        this.ctx.fillStyle = 'rgb(0, 0, 0)';
        this.ctx.fillRect(0, 0, this.canvasWidth, this.canvasHeight);
    }
    updateScreen() {
        const clientRatio = this.canvasWidth / this.canvasHeight;
        const screenRatio = this.screen.width / this.screen.height;
        if (screenRatio > clientRatio) {
            this.screenWidth = this.canvasWidth - 2;
            this.screenHeight = Math.floor(this.canvasWidth / screenRatio);
            this.screenOffsetX = 1;
            this.screenOffsetY = Math.round((this.canvasHeight - this.screenHeight) / 2);
        }
        else {
            this.screenWidth = Math.round(this.canvasHeight * screenRatio);
            this.screenHeight = this.canvasHeight - 2;
            this.screenOffsetY = 1;
            this.screenOffsetX = Math.round((this.canvasWidth - this.screenWidth) / 2);
        }
        this.ctx.strokeStyle = 'rgb(0, 0, 255)';
        this.ctx.rect(this.screenOffsetX, this.screenOffsetY, this.screenWidth, this.screenHeight);
        this.ctx.stroke();
    }
    updateDisplay() {
        if (!this.sequence) {
            return;
        }
        console.log(this.sequence);
        const screenScaleX = this.screenWidth / this.screen.width;
        const screenScaleY = this.screenHeight / this.screen.height;
        this.displayWidth = Math.round(this.width * screenScaleX);
        this.displayHeight = Math.round(this.height * screenScaleY);
        this.displayOffsetX = this.screenOffsetX + Math.round(this.offsetX * screenScaleX);
        this.displayOffsetY = this.screenOffsetY + Math.round(this.offsetY * screenScaleY);
        this.ctx.fillStyle = 'rgb(0, 0, 0)';
        this.ctx.fillRect(this.displayOffsetX, this.displayOffsetY, this.displayWidth, this.displayHeight);
        console.log(`${this.displayOffsetX}, ${this.displayOffsetY}, ${this.displayWidth}, ${this.displayHeight}`);
        this.updateImage();
    }
    updateImage() {
        const img = new Image;
        img.onload = function () {
            this.ctx.drawImage(img, this.displayOffsetX, this.displayOffsetY, this.displayWidth, this.displayHeight);
        }.bind(this);
        img.src = `/${this.displayWidth}/${this.displayHeight}/image.jpg?cacheBreaker=${+new Date()}`;
    }
    update(msg) {
    }
    set(state) {
        this.sequence = true;
        this.offsetX = state.offset.x;
        this.offsetY = state.offset.y;
        this.width = state.display.width;
        this.height = state.display.height;
        this.updateDisplay();
    }
    onResize(event) {
        this.updateSize();
        this.clear();
        this.updateScreen();
        this.updateDisplay();
    }
}
class Client {
    constructor() {
        this.connected = false;
        this.frames = 0;
        let uri = this.getWebsocketUri();
        this.progress = document.getElementById('progress');
        this.progressText = document.getElementById('progressText');
        this.client = new WebSocket(uri);
        this.display = new Display();
        this.client.onopen = this.onOpen.bind(this);
        this.client.onclose = this.onClose.bind(this);
        this.client.onmessage = this.onMessage.bind(this);
        document.getElementById('sequenceSelectForm').reset();
        document.getElementById('sequenceCtrlForm').reset();
        document.getElementById('manualCtrlForm').reset();
        this.disableClass('sequenceCtrl');
        this.disableClass('manualCtrl');
        this.setProgress({ hash: null, progress: 0 });
    }
    getWebsocketUri() {
        const host = (window.location.host + '').split(':')[0];
        return `ws://${host}:${WEBSOCKET_PORT}`;
    }
    onMessage(event) {
        const msg = JSON.parse(event.data);
        if (typeof msg.cmd !== 'undefined' && msg.cmd !== null) {
            this.cmd(msg);
        }
    }
    onOpen(event) {
        console.log('Connected');
        this.connected = true;
        this.active();
        this.enableClass('manualCtrl');
    }
    onClose(event) {
        console.log('Disconnected');
        this.connected = false;
        this.inactive();
    }
    setSequence(state) {
        this.setProgress(state.sequence);
        this.setFrame(state.sequence);
        this.setStatus(state.sequence);
        this.setDisplay(state);
        document.getElementById('sequence').value = state.sequence.hash;
    }
    setUpdate(state) {
        this.setProgress(state.sequence);
        this.setFrame(state.sequence);
        this.setStatus(state.sequence);
        this.display.updateImage();
    }
    setStatus(sequence) {
        let status;
        switch (sequence.status) {
            case SequenceStatus.IDLE:
                status = 'Idle';
                break;
            case SequenceStatus.RUNNING:
                status = 'Running';
                break;
            case SequenceStatus.PAUSED:
                status = 'Paused';
                break;
            default:
                status = 'Unknown State';
        }
        this.frames = sequence.frames;
        document.getElementById('sequenceStatus').innerText = status;
        document.getElementById('sequenceName').innerText = sequence.name;
        document.getElementById('sequenceLength').innerText = `Sequence Length: ${sequence.frames}`;
    }
    setProgress(sequence) {
        const percent = sequence.progress * 100.0;
        if (this.progress !== null) {
            this.progress.value = sequence.progress;
            this.progressText.innerText = `Progress: ${Math.floor(percent)}%`;
        }
    }
    setFrame(sequence) {
        if (typeof sequence.current !== 'undefined') {
            document.getElementById('frame').value = `${sequence.current}`.padStart(5, '0');
        }
    }
    setDisplay(state) {
        const widthEl = document.getElementById('displayWidth');
        const heightEl = document.getElementById('displayHeight');
        const srcWidthEl = document.getElementById('sourceWidth');
        const srcHeightEl = document.getElementById('sourceHeight');
        if (typeof state.display !== 'undefined') {
            widthEl.value = state.display.width;
            heightEl.value = state.display.height;
            srcWidthEl.value = state.source.width;
            srcHeightEl.value = state.source.height;
            widthEl.readOnly = false;
            heightEl.readOnly = false;
            this.display.set(state);
        }
    }
    cmd(msg) {
        switch (msg.cmd) {
            case 'ping':
                this.receivePing();
                break;
            case 'open':
                this.receiveCameraOpen();
                break;
            case 'close':
                this.receiveCameraClose();
                break;
            case 'select':
                this.receiveSelect(msg);
                break;
            case 'update':
                this.receiveUpdate(msg);
                break;
            default:
                console.warn(`No command "${msg.cmd}"`);
                break;
        }
    }
    disableClass(className) {
        console.log(`Disabling class: ${className}`);
        document.querySelectorAll(`.${className}`).forEach((el) => {
            el.disabled = true;
        });
    }
    enableClass(className) {
        console.log(`Enabling class: ${className}`);
        document.querySelectorAll(`.${className}`).forEach((el) => {
            el.disabled = false;
        });
    }
    sendCameraOpen() {
        console.log('send camera open');
        this.disableClass('manualCtrl');
        this.client.send(JSON.stringify({ cmd: 'open' }));
    }
    receivePing() {
        this.sendPong();
    }
    receiveCameraOpen() {
        console.log('got camera open');
        this.enableClass('manualCtrl');
    }
    sendCameraClose() {
        console.log('send camera close');
        this.disableClass('manualCtrl');
        this.client.send(JSON.stringify({ cmd: 'close' }));
    }
    receiveCameraClose() {
        console.log('got camera close');
        this.enableClass('manualCtrl');
    }
    sendPong() {
        this.client.send(JSON.stringify({ cmd: 'pong' }));
    }
    sendAdvance() {
        this.client.send(JSON.stringify({ cmd: 'advance' }));
    }
    sendRewind() {
        this.client.send(JSON.stringify({ cmd: 'rewind' }));
    }
    sendFrameSet() {
        const frameStr = document.getElementById('frame').value;
        let frameNum = null;
        try {
            frameNum = parseInt(frameStr);
        }
        catch (err) {
            console.error(`Error parsing ${frameStr}`);
        }
        if (frameNum === null) {
            frameNum = 0;
        }
        if (frameNum < 0) {
            frameNum = 0;
        }
        if (frameNum > this.frames - 1) {
            frameNum = this.frames - 1;
        }
        this.client.send(JSON.stringify({ cmd: 'set', state: { sequence: { current: frameNum } } }));
    }
    sendToEnd() {
        this.client.send(JSON.stringify({ cmd: 'set', state: { sequence: { current: this.frames - 1 } } }));
    }
    sendToStart() {
        this.client.send(JSON.stringify({ cmd: 'set', state: { sequence: { current: 0 } } }));
    }
    sendSelect() {
        const hash = document.getElementById('sequence').value;
        let msg;
        if (hash === '- Select Image Sequence -') {
            return;
        }
        msg = { cmd: 'select', state: { sequence: { hash } } };
        console.log('send select');
        console.log(hash);
        this.client.send(JSON.stringify(msg));
    }
    receiveSelect(msg) {
        console.log('got select');
        this.enableClass('sequenceCtrl');
        this.setSequence(msg.state);
    }
    sendStart() {
        this.client.send(JSON.stringify({ cmd: 'start' }));
    }
    sendStop() {
        this.client.send(JSON.stringify({ cmd: 'stop' }));
    }
    receiveUpdate(msg) {
        this.setUpdate(msg.state);
    }
    fullscreen() {
        if (!document.fullscreenElement) {
            document.documentElement.requestFullscreen();
        }
        else {
            this.exitFullscreen();
        }
    }
    exitFullscreen() {
        if (document.fullscreenElement) {
            document.exitFullscreen();
        }
    }
    active() {
        document.getElementById('overlay').classList.add('active');
    }
    inactive() {
        document.getElementById('overlay').classList.remove('active');
    }
}
const client = new Client();
//# sourceMappingURL=index.js.map